/********************************************************************
 FileName:     	HardwareProfile - PIC18F26J50 Demo.h
 Dependencies:	See INCLUDES section
 Processor:	PIC18 USB Microcontrollers
 Hardware:	PIC18F26J50 Demo
 Compiler:  	Microchip C18
 Company:	Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the CompanyE for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   11/19/2004   Initial release
  2.1   02/26/2007   Updated for simplicity and to use common
                     coding style
  2.3   09/15/2008   Broke out each hardware platform into its own
                     "HardwareProfile - xxx.h" file
  2.4	03/11/2009   Slight update
********************************************************************/

#ifndef HARDWARE_PROFILE_PIC18F26J50_DEMO_H
#define HARDWARE_PROFILE_PIC18F26J50_DEMO_H

    /*******************************************************************/
    /******** USB stack hardware selection options *********************/
    /*******************************************************************/
    //This section is the set of definitions required by the MCHPFSUSB
    //  framework.  These definitions tell the firmware what mode it is
    //  running in, and where it can find the results to some information
    //  that the stack needs.
    //These definitions are required by every application developed with
    //  this revision of the MCHPFSUSB framework.  Please review each
    //  option carefully and determine which options are desired/required
    //  for your application.

    //#define USE_SELF_POWER_SENSE_IO
    #define tris_self_power     TRISCbits.TRISC2    // Input
    #define self_power          1

//    #define USE_USB_BUS_SENSE_IO
    #define tris_usb_bus_sense  TRISAbits.TRISA2    // Input
    #if defined(USE_USB_BUS_SENSE_IO)
    #define USB_BUS_SENSE       PORTAbits.RA2
    #else
    #define USB_BUS_SENSE       1
    #endif
 
    //Uncomment this to make the output HEX of this project 
    //   to be able to be bootloaded using the HID bootloader
	#define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER		

    /*******************************************************************/
    /******** MDD File System selection options ************************/
    /*******************************************************************/
    #define USE_PIC18

    #define USE_SD_INTERFACE_WITH_SPI

    #define TRIS_CARD_DETECT    TRISBbits.TRISB0    // Input
    #define CARD_DETECT         PORTBbits.RB0
    
//     #define TRIS_WRITE_DETECT   TRISDbits.TRISD7    // Input
//     #define WRITE_DETECT        PORTDbits.RD7
    #define TRIS_WRITE_DETECT   TRISBbits.TRISB0    // Input
    #define WRITE_DETECT        PORTBbits.RB0
 
    // Chip Select Signal
    #define SD_CS               PORTBbits.RB3
    #define SD_CS_TRIS          TRISBbits.TRISB3
        
    // Card detect signal
    #define SD_CD               PORTBbits.RB0
    #define SD_CD_TRIS          TRISBbits.TRISB0
        
    // Write protect signal
//     #define SD_WE               PORTDbits.RD7
//     #define SD_WE_TRIS          TRISDbits.TRISD7
     #define SD_WE               PORTBbits.RB0
     #define SD_WE_TRIS          TRISBbits.TRISB0

    // TRIS pins for the SCK/SDI/SDO lines
    #define SPICLOCK            TRISBbits.TRISB1
    #define SPIIN               TRISAbits.TRISA1
    #define SPIOUT              TRISAbits.TRISA5

    // Latch pins for SCK/SDI/SDO lines
    #define SPICLOCKLAT         LATBbits.LATB1
    #define SPIINLAT            LATAbits.LATA1
    #define SPIOUTLAT           LATAbits.LATA5

    // Port pins for SCK/SDI/SDO lines
    #define SPICLOCKPORT        PORTBbits.RB1
    #define SPIINPORT           PORTAbits.RA1
    #define SPIOUTPORT          PORTAbits.RA5

    // Registers for the SPI module you want to use
    #define SPICON1             SSP2CON1
    #define SPISTAT             SSP2STAT
    #define SPIBUF              SSP2BUF
    #define SPISTAT_RBF         SSP2STATbits.BF
    #define SPICON1bits         SSP2CON1bits
    #define SPISTATbits         SSP2STATbits

    #define SPI_INTERRUPT_FLAG  PIR3bits.SSP2IF 
    #define SPI_INTERRUPT_FLAG_ASM  PIR3, 7 
    #define SPIENABLE           SSP2CON1bits.SSPEN

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

    /** Board definition ***********************************************/
    //These defintions will tell the main() function which board is
    //  currently selected.  This will allow the application to add
    //  the correct configuration bits as wells use the correct
    //  initialization functions for the board.  These defitions are only
    //  required in the stack provided demos.  They are not required in
    //  final application design.
    #define DEMO_BOARD PIC18F26J50_DEMO
    #define PIC18F26J50_DEMO
    #define CLOCK_FREQ 48000000
    #define GetSystemClock() CLOCK_FREQ   
    #define GetInstructionClock() GetSystemClock()

    /** LED ************************************************************/
// #define mInitAllLEDs()      LATA &= 0xF3; TRISA &= 0xF3;
// 
// #define mLED_1              LATAbits.LATA2
// #define mLED_2              LATAbits.LATA3

	#define mInitAllLEDs()      LATB &= 0x3F; TRISB &= 0x3F;

	#define mLED_1              LATBbits.LATB6
	#define mLED_2              LATBbits.LATB7

    #define mLED_3              
    #define mLED_4              
    
    #define mGetLED_1()         mLED_1
    #define mGetLED_2()         mLED_2
    #define mGetLED_3()         1
    #define mGetLED_4()         1

    #define mLED_1_On()         mLED_1 = 1;
    #define mLED_2_On()         mLED_2 = 1;
    #define mLED_3_On()         
    #define mLED_4_On()        
    
    #define mLED_1_Off()        mLED_1 = 0;
    #define mLED_2_Off()        mLED_2 = 0;
    #define mLED_3_Off()        
    #define mLED_4_Off()        
    
    #define mLED_1_Toggle()     mLED_1 = !mLED_1;
    #define mLED_2_Toggle()     mLED_2 = !mLED_2;
    #define mLED_3_Toggle()     
    #define mLED_4_Toggle() 
    
    /** SWITCH *********************************************************/
    #define mInitSwitch2()      TRISBbits.TRISB2=1;
    #define mInitSwitch3()      mInitSwitch2();
    #define mInitAllSwitches()  mInitSwitch2();
    #define sw2                 PORTBbits.RB2
    #define sw3                 PORTBbits.RB2

    /** POT ************************************************************/
    #define mInitPOT()          {TRISAbits.TRISA0=1;			\
    							 ANCON0bits.PCFG0 = 0;			\
    							 ADCON0=0x01;					\
    							 ADCON1=0x9E;}		// POT on RA0

    /** I/O pin definitions ********************************************/
    #define INPUT_PIN 1
    #define OUTPUT_PIN 0

#endif  //HARDWARE_PROFILE_PIC18F26J50_DEMO_H
