/********************************************************************
 FileName:		main.c
 Dependencies:	See INCLUDES section   << HID Bootloade >>
 Processor:		PIC18 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 Complier:  	Microchip C18 (for PIC18)
 Company:		Copyright (c) 2012 iwamoto All Rights Reserved

********************************************************************
 File Description:
			SWƎMVAf[^SDɕۑ
 Change History:
  Rev   Description
  ----  -----------------------------------------
 
********************************************************************/

/** INCLUDES *******************************************************/

#include <stdio.h>

#include "USB/USB.h"
#include "HardwareProfile.h"
#include "MDD File System/SD-SPI.h"
#include "MDD File System/FSIO.h"

#include "./USB/usb_function_msd.h"

#define CR_EN RCSTA1bits.CREN
#define O_ERR RCSTA1bits.OERR
#define MonCnt    10                // `^O̖h~

/** CONFIGURATION **************************************************/
     #pragma config WDTEN = OFF          //WDT disabled (enabled by SWDTEN bit)
     #pragma config PLLDIV = 2           //Divide by 2 (8 MHz internal oscillator input)
     #pragma config STVREN = ON          //stack overflow/underflow reset enabled
     #pragma config XINST = OFF          //Extended instruction set disabled
     #pragma config CPUDIV = OSC1        //No CPU system clock divide
     #pragma config CP0 = OFF            //Program memory is not code-protected
     #pragma config OSC = INTOSCPLL      //Internal oscillator, PLL enabled, used by USB
     #pragma config T1DIG = OFF          //Sec Osc clock source may not be selected, unless T1OSCEN = 1
     #pragma config LPT1OSC = OFF        //high power Timer1 mode
     #pragma config FCMEN = OFF          //Fail-Safe Clock Monitor disabled
     #pragma config IESO = OFF           //Two-Speed Start-up disabled
     #pragma config WDTPS = 32768        //1:32768
     #pragma config DSWDTOSC = INTOSCREF //DSWDT uses INTOSC/INTRC as clock
     #pragma config RTCOSC = T1OSCREF    //RTCC uses T1OSC/T1CKI as clock
     #pragma config DSBOREN = OFF        //Zero-Power BOR disabled in Deep Sleep
     #pragma config DSWDTEN = OFF        //Disabled
     #pragma config DSWDTPS = 8192       //1:8,192 (8.5 seconds)
     #pragma config IOL1WAY = OFF        //IOLOCK bit can be set and cleared
     #pragma config MSSP7B_EN = MSK7     //7 Bit address masking
     #pragma config WPFP = PAGE_1        //Write Protect Program Flash Page 0
     #pragma config WPEND = PAGE_0       //Start protection at page 0
     #pragma config WPCFG = OFF          //Write/Erase last page protect Disabled
     #pragma config WPDIS = OFF          //WPFP[5:0], WPEND, and WPCFG bits ignored 

/** VARIABLES ******************************************************/
#pragma udata
// ---------- ʕϐ -------------------------
char TempBuf[100];			// bobt@
char PtrBuf = 0;			// ݃|C^
char RxBuf[100];			// Mobt@
char RxSize = 0;			// M
unsigned char T2Tick = 0;  // Timer2݂̉
char TaskSec=0;            // bTaskFlag
char SWcount = MonCnt;      // `^Oh~̃JEg
char LoggingON = 0;         // Logw}[N
char LED1cnt, LED2cnt = 0;         // LED2 _ԃ}[N

/* Standard Response to INQUIRY command stored in ROM 	*/
const ROM InquiryResponse inq_resp = {
	0x00,		// peripheral device is connected, direct access block device
	0x80,           // removable
	0x04,	 	// version = 00=> does not conform to any standard, 4=> SPC-2
	0x02,		// response is in format specified by SPC-2
	0x20,		// n-4 = 36-4=32= 0x20
	0x00,		// sccs etc.
	0x00,		// bque=1 and cmdque=0,indicates simple queueing 00 is obsolete,
			// but as in case of other device, we are just using 00
	0x00,		// 00 obsolete, 0x80 for basic task queueing
	{'M','i','c','r','o','c','h','p'},
	// this is the T10 assigned Vendor ID
	{'M','a','s','s',' ','S','t','o','r','a','g','e',' ',' ',' ',' '},
	{'0','0','0','1'}
};

/** PRIVATE PROTOTYPES *********************************************/
static void InitializeSystem(void);
void USBDeviceTasks(void);
void ProcessIO(void);
WORD_VAL ReadPOT(void);
void YourHighPriorityISRCode(void);
void YourLowPriorityISRCode(void);
void USBCBSendResume(void);

/** VECTOR REMAPPING ***********************************************/
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
	
	extern void _startup (void);
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void){
	    _asm goto _startup _endasm
	}
	#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
	void Remapped_High_ISR (void)	{
	     _asm goto YourHighPriorityISRCode _endasm
	}
	#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
	void Remapped_Low_ISR (void)	{
	     _asm goto YourLowPriorityISRCode _endasm
	}
	
	#pragma code HIGH_INTERRUPT_VECTOR = 0x08
	void High_ISR (void){
	     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#pragma code LOW_INTERRUPT_VECTOR = 0x18
	void Low_ISR (void){
	     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
	}

	#pragma code
	
	//These are your actual interrupt handling routines.
	#pragma interrupt YourHighPriorityISRCode
	void YourHighPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
        #if defined(USB_INTERRUPT)
	        USBDeviceTasks();
        #endif
	
	}	//This return will be a "retfie fast", since this is in a #pragma interrupt section 
	#pragma interruptlow YourLowPriorityISRCode
	void YourLowPriorityISRCode()
	{
	    char RxData;
		if(PIR1bits.RC1IF){
		    if(O_ERR){
				CR_EN=0;                // OverrunG[NA
				CR_EN=1;
			}else{
				RxData = RCREG1;		// C^vgtbONA
				LED2cnt = 4;							// LED20mS_w
				TempBuf[PtrBuf++] = RxData;				// i[
				if(RxData == 0x0A){						// f[^I
					memcpy(RxBuf, TempBuf, PtrBuf);		// obt@Rs[
					RxSize = PtrBuf;					// MoCgۑ
					PtrBuf = 0;							// |C^Zbg
				}
				if(PtrBuf >= 100)PtrBuf = 0;			// eʂ𒴂ꍇׂĔp
			}
		}	// UART RxIF
		if(PIR1bits.TMR2IF){		// 5mSɊ
			PIR1bits.TMR2IF = 0;		// ݃tbONA

			//***************LED莞ԓ_************************
			// LED莞ԓ_
			//******************************************************
		    if(LED1cnt){                          // LED2_wȂ
				mLED_1_On();                      // LED2_
			    if(--LED1cnt == 0) mLED_1_Off();	// ԏI
			}
		    if(LED2cnt){                          // LED2_wȂ
				mLED_2_On();                      // LED2_
			    if(--LED2cnt == 0) mLED_2_Off();	// ԏI
			}
			//***************`^Oh~************************
			// 10mS̊݃[`ɃvO
			// pushSWmɗĂƂɁASW
			// `^Oruꂽv}[N
			//    LoggingON    ]
			//******************************************************
		    if(SWcount){                // `^Oh~Ȃ
		        if(sw2)SWcount--;       //   OFF̌ph~ԑ҂
		        else SWcount = MonCnt;  //   ONȂAԉ
		    }else{                      // h~I
		        if(!sw2){               //   SWꂽ
		            LoggingON = !LoggingON;  // uLogwv}[NP
		            SWcount = MonCnt;        //     `^Oh~ԂZbg
		        }
		    }

// 			mLED_2_Off();				// LED2_
// 			if(T2Tick++ > 200){		    // 1b
// 				T2Tick = 0;		        // 
// 			    TaskSec=1;              // bTaskFlagZbg
// 				mLED_2_On();			// LED2
// 			}

		}
	}	//This return will be a "retfie", since this is in a #pragma interruptlow section 


/** DECLARATIONS ***************************************************/
#pragma code

/********************************************************************
 * Function:        void main(void)
 *******************************************************************/
void main(void){   
	char j;

    unsigned int* data;
    unsigned int i;
    FSFILE *logFile;
    BOOL initResults;
    WORD_VAL ADCResult;
    BYTE printData[6]="123\r\n";

    //set the data pointer for the FSfwrite function
    data = &i;
    InitializeSystem();
    initResults = FSInit();
    
    mInitPOT();         //initialize the A/D
    mInitAllLEDs();    //Set up the LED as output
    mInitSwitch2();    //Set the push button input

    //The log file is not open so let's make sure the pointer is NULLed out
    logFile = NULL;

    USBDeviceInit();
    USBDeviceAttach();    // Interrupt method.

    while(1){
        ProcessIO();        // See msd.c & msd.h

        //USBʐMm̂߂̊Ԃ́A̓ɐO邽
        // Ԃ̂ƂsȂ

        if((USBDeviceState < CONFIGURED_STATE) && (PORTAbits.RA2 == 1)) {
            continue;       //  while(1){ ɖ߂
        }

        if(initResults == TRUE){     // FSInit()̌ʂQ

        //USB  MDD Ƀt@CɃANZXȂ悤ɂ
        //ȒPȕ@́AUSBڑĂԁAfAvA
        //t@Cɏ܂Ȃ悤邱ƂłB
        //if(USBDeviceState < DEFAULT_STATE)
        //{
            if(LoggingON){       // LOGw@ON
                //PCt@Cɏ܂Ȃ悤 soft detach 
                //  Ă܂B
                LUNSoftDetach(0);
                
                if(logFile != NULL){       // Ƀt@CI[v
                    // Get an ADC sample      // ADϊ
//                     if(TaskSec){
// 						TaskSec=0;
// 	                    ADCResult = ReadPOT();
// 	                    sprintf((char*)&printData[0],"%d\r\n",ADCResult.Val);
// 	                    FSfwrite((const void*)&printData[0],1,sizeof(printData),logFile);
//                     }
			        if(RxSize){                            // M
						FSfwrite(RxBuf,1,RxSize,logFile);
						RxSize = 0;
					}
                }else{                           // I[vĂȂȂ
                        logFile = FSfopenpgm("log.csv",WRITE);    // I[v
                }
                mLED_1_On();                     // LED\ON
            }else{          // SWOFFȂ
                if(logFile != NULL){       // I[vĂȂ
                    FSfclose(logFile);     // N[Y
                    logFile = NULL;
                    //݂Î reattach s
                    //  PC 珑񂾃t@Cǂ߂悤ɂ܂
                    LUNSoftAttach(0);     // PCɃAbNZX
                }
                mLED_1_Off();                     // LED\OFF
            }
        //}
        }else{       // FSInit()̌ʂFailȂēxFSInit()s
            if(MDD_SDSPI_MediaDetect() == TRUE){ 
                initResults = FSInit();
            }
        }
    } //end while
    
}//end main

/********************************************************************
 * Function:        static void InitializeSystem(void)
 *******************************************************************/
static void InitializeSystem(void){
    unsigned int pll_startup_counter = 600;
    OSCTUNEbits.PLLEN = 1;          // PLL ғ
    while(pll_startup_counter--);   // PLL lock܂ 2+ms ҂

    ANCON0 = 0xFF;                  // fW^
    ANCON1 = 0xFF;                  // 

//	HardwareProfile.h "USE_USB_BUS_SENSE_IO" @\gpƐ錾ꍇ
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN;
    #endif
    
//	HardwareProfile.h."USE_SELF_POWER_SENSE_IO" @\gpƐ錾ꍇ
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;
    #endif

    //********* Initialize Peripheral Pin Select (PPS) *************************
    //Initialize the SPI
    RPINR21 = 1;   //SDI = RP1
    RPOR4 = 10;    //RP4 = SCK
    RPOR2 = 9;     //RP2 = SDO
    RPINR22 = 4;   //SCK = RP4

    //card detect̎ pull-up Lɂ SD-Card ̖}ɔ
    INTCON2bits.RBPU = 0; 

    T2CON = 0b01110110;
	PR2 = 249;
	PIR1bits.TMR2IF=0;	// ^C}[Q tO  
	PIE1bits.TMR2IE=1;	//          
	IPR1bits.TMR2IP=0;	// ݗDx   

	TRISCbits.TRISC7 = 1;	// UART Rx Input
	TRISCbits.TRISC6 = 0;	// UART Tx Output
	RCSTA1 = 0b10010000;	// 񓯊WBit 
	TXSTA1 = 0b10100110;	// BRGH_HIGH
	BAUDCON1 = 0b11001000;	// 16_BIT_RATE
	SPBRGH1 = 0x04;			// 9600bps
	SPBRG1 = 0xE1;			// 0x04E1 = 1249
	IPR1bits.RC1IP = 0;		// DC^v
	PIE1bits.RC1IE = 1;		// C^vg

	RCONbits.IPEN=1;		// QiK̃C^vgɐݒ
	INTCONbits.GIEH=1;		// DC^vg
	INTCONbits.GIEL=1;		// DC^vg

}//end InitializeSystem


/********************************************************************
 * Function:        void ProcessIO(void)
 *******************************************************************/
void ProcessIO(void){   

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
    //  USBڑ̏ꍇ
    MSDTasks();    
}//end ProcessIO

/******************************************************************************
 * Function:        WORD_VAL ReadPOT(void)
 *****************************************************************************/
WORD_VAL ReadPOT(void){
    WORD_VAL w;
        mInitPOT();
        ADCON0bits.GO = 1;              // Start AD conversion
        while(ADCON0bits.NOT_DONE);     // Wait for conversion
        w.v[0] = ADRESL;
        w.v[1] = ADRESH;
    return w;
}//end ReadPOT

// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *****************************************************************************/
void USBCBSuspend(void){
}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
    #if !defined(self_powered)
        if(U1OTGIRbits.ACTVIF)
        {
            IEC5bits.USB1IE = 0;
            U1OTGIEbits.ACTVIE = 0;
            IFS5bits.USB1IF = 0;
        
            //USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
            USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
            //USBSuspendControl = 0;
        }
    #endif
}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
}

/********************************************************************
 Function:        void USBCB_SOF_Handler(void)
 *******************************************************************/
void USBCB_SOF_Handler(void){
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *******************************************************************/
void USBCBErrorHandler(void){
}

/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *******************************************************************/
void USBCBCheckOtherReq(void){
    USBCheckMSDRequest();
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *******************************************************************/
void USBCBStdSetDscHandler(void){
    // Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *******************************************************************/
void USBCBInitEP(void){
    #if (MSD_DATA_IN_EP == MSD_DATA_OUT_EP)
        USBEnableEndpoint(MSD_DATA_IN_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    #else
        USBEnableEndpoint(MSD_DATA_IN_EP,USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
        USBEnableEndpoint(MSD_DATA_OUT_EP,USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    #endif

    USBMSDInit();
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *******************************************************************/
void USBCBSendResume(void){
    static WORD delay_count;

    if(USBGetRemoteWakeupStatus() == TRUE) {
        //Verify that the USB bus is in fact suspended, before we send
        //remote wakeup signalling.
        if(USBIsBusSuspended() == TRUE) {
            USBMaskInterrupts();
            
            //Clock switch to settings consistent with normal USB operation.
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FALSE;  //So we don't execute this code again, 

            delay_count = 3600U;        
            do {
                delay_count--;
            }while(delay_count);
            
            //Now drive the resume K-state signalling onto the USB bus.
            USBResumeControl = 1;       // Start RESUME signaling
            delay_count = 1800U;        // Set RESUME line for 1-13 ms
            do{
                delay_count--;
            }while(delay_count);
            USBResumeControl = 0;       //Finished driving resume signalling

            USBUnmaskInterrupts();
        }
    }
}
rom const char copyr[] = {73,87,65,32,83,79,70,84}; 

/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_TRANSFER:
            //Add application specific callback task or callback function here if desired.
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER_TERMINATED:

            if(MSDWasLastCBWValid() == FALSE)  {
                //Need to re-stall the endpoints, for persistent STALL behavior.
    			USBStallEndpoint(MSD_DATA_IN_EP, IN_TO_HOST);
      			USBStallEndpoint(MSD_DATA_OUT_EP, OUT_FROM_HOST);                 
            }else{   
                //Check if the host cleared halt on the bulk out endpoint.  In this
                //case, we should re-arm the endpoint, so we can receive the next CBW.
                if((USB_HANDLE)pdata == USBGetNextHandle(MSD_DATA_OUT_EP, OUT_FROM_HOST))
                {
                    USBMSDOutHandle = USBRxOnePacket(MSD_DATA_OUT_EP, (BYTE*)&msd_cbw, MSD_OUT_EP_SIZE);
                }    
            }    
            break;
        default:
            break;
    }      
    return TRUE; 
}

           
/** EOF main.c ***************************************************************/

